@@ -261,21 +261,43 @@ def Range(limit: AST, start: AST = Integer(0), step: AST = Integer(1)):
     )
 
 
+def MapList(l: AST, m: AST = Lambda(["x"], Var("x"))):
+    """Apply a map function on each element in a list"""
+    return Apply(
+        Lambda(
+            ["op"],
+            RecFun(
+                Lambda(
+                    ["map", "xs"],
+                    Ite(
+                        NullList(Var("xs")),
+                        EmptyList(),
+                        PrependList(
+                            Apply(Var("op"), HeadList(Var("xs"))),
+                            Apply(
+                                Var("map"),
+                                Var("map"),
+                                TailList(Var("xs")),
+                            ),
+                        ),
+                    ),
+                ),
+            ),
+        ),
+        m,
+        l,
+    )
+
+
 # Data Utils
 
 
 def Constructor(d: AST):
-    return Apply(
-        Force(Force(BuiltIn(uplc_ast.BuiltInFun.FstPair))),
-        Apply(BuiltIn(uplc_ast.BuiltInFun.UnConstrData), d),
-    )
+    return FstPair(UnConstrData(d))
 
 
 def Fields(d: AST):
-    return Apply(
-        Force(Force(BuiltIn(uplc_ast.BuiltInFun.SndPair))),
-        Apply(BuiltIn(uplc_ast.BuiltInFun.UnConstrData), d),
-    )
+    return SndPair(UnConstrData(d))
 
 
 def NthField(d: AST, n: AST):
